#define ZASM_EXPR_C 1

#include <zasm/alloc.h>
#include <zasm/expr.h>
#include <zasm/memory.h>
#include <zasm/string.h>

ALLOC_DEFINE_DELETE(expr, ALLOC_DEF_COUNT);

struct expr_t *expr_new(struct expr_alloc_t **alloc)
{
  struct expr_t *val;

  val = expr_alloc_get(alloc);
  expr_fill(val);
  return val;
}

void expr_fill(struct expr_t *expr)
{}

int expr_calculate(struct expr_t *expr)
{
  int a, b, c;
  if (!expr)
    return 0;
  switch (expr->type)
    {
    case EXPR_OPERATOR:
      a = expr_calculate(expr->value.function.arg1);
      b = expr_calculate(expr->value.function.arg2);
      c = expr_calculate(expr->value.function.arg3);
      return expr->value.function.calc(a,b,c);
    case EXPR_NUMBER:
      return expr->value.value;
    case EXPR_NAME:
      ERROR("name not yet known: %s\n", expr->value.name.name);
    default:
      ERROR("internal error\n");
    }
  return 0;
}

int expr_link_calculate(struct expr_t *expr, struct link_namespace_t *ns)
{
  struct link_entry_t *e;
  int a, b, c;
  if (!expr)
    return 0;
  switch (expr->type)
    {
    case EXPR_OPERATOR:
      a = expr_link_calculate(expr->value.function.arg1, ns);
      b = expr_link_calculate(expr->value.function.arg2, ns);
      c = expr_link_calculate(expr->value.function.arg3, ns);
      return expr->value.function.calc(a,b,c);
    case EXPR_NUMBER:
      return expr->value.value;
    case EXPR_NAME:
      e = link_namespace_lookup_direct(ns, expr->value.name.name);
      if (!e)
        ERROR("label not yet known: \"%s\"\n", expr->value.name.name);
      return link_expr_calculate(e->expr, ns);
    default:
      ERROR("internal error\n");
    }
  return 0;
}

void expr_delete(struct expr_t *expr)
{
  if (!expr)
    return;

  switch (expr->type)
    {
    case EXPR_OPERATOR:
      //      expr_delete(expr->value.function.arg1);
      //      expr_delete(expr->value.function.arg2);
      //      expr_delete(expr->value.function.arg3);
      break;
    case EXPR_NAME:
      string_free(expr->value.name.name);
      string_free(expr->value.name.part);
      break;
    default:
      break;
    }
}

void expr_free(struct expr_t *expr)
{
  expr_delete(expr);
}
