#define ZASM_STRING_ALLOC_C 1

#include <zasm/string_alloc.h>
#include <string.h>
#include <zasm/memory.h>

struct string_alloc_t *string_alloc_new()
{
  struct string_alloc_t *val;
  
  ALLOCATE(struct string_alloc_t, val, 1);
  string_alloc_fill(val);
  return val;
}

void string_alloc_fill(struct string_alloc_t *alloc)
{
  alloc_fill(&(alloc->alloc));
}

void string_alloc_add(struct string_alloc_t **alloc, char *string)
{
  struct string_alloc_t *temp;

  if ((!(*alloc)) || ((*alloc)->alloc.i >= ALLOC_DEF_COUNT))
    {
      temp = string_alloc_new();
      temp->alloc.previous = &((*alloc)->alloc);
      *alloc = temp;
    }
  (*alloc)->data[(*alloc)->alloc.i] = string;
  (*alloc)->alloc.i++;
}

char *string_alloc_copy(struct string_alloc_t **alloc, char *string)
{
  string_alloc_add(alloc, string);
  return string_copy(string);
}

void string_alloc_reset(struct string_alloc_t *alloc)
{
  string_alloc_delete(alloc);
  string_alloc_fill(alloc);
}

void string_alloc_delete(struct string_alloc_t *alloc)
{
  int i;

  for (i = 0; i < alloc->alloc.i; i++)
    string_free(alloc->data[i]);
  if (alloc->alloc.previous)
    string_alloc_free(STRING_ALLOC_T(alloc->alloc.previous));
}

void string_alloc_free(struct string_alloc_t *alloc)
{
  if (!alloc)
    return;
  string_alloc_delete(alloc);
  free(alloc);
}
