#ifndef ZASM_TOKENS_H
#define ZASM_TOKENS_H 1

#include <stdio.h>

#define TOK_COLON TOK_OP_ELSE

enum
  {
TOK_MULT_LIST,
TOK_PROCESSOR,
TOK_PROC_INSTR,
TOK_PROC_INSTR_ARG,
TOK_LITTLE_ENDIAN,
TOK_BIG_ENDIAN,
TOK_ADDINSTR,
TOK_BYTE_INT,
TOK_WORD_INT,
TOK_DWORD_INT,
TOK_BYTE_INT_P,
TOK_WORD_INT_P,
TOK_DWORD_INT_P,
TOK_SIGNED_BYTE_INT,
TOK_SIGNED_WORD_INT,
TOK_SIGNED_DWORD_INT,
TOK_SIGNED_BYTE_INT_P,
TOK_SIGNED_WORD_INT_P,
TOK_SIGNED_DWORD_INT_P,
TOK_BYTE,
TOK_WORD,
TOK_DWORD,
TOK_SIGNED_BYTE,
TOK_SIGNED_WORD,
TOK_SIGNED_DWORD,
TOK_ARG,
TOK_DB,
TOK_DW,
TOK_DD,
TOK_NONE,
TOK_LIST_END,
TOK_STREAM_END,
TOK_END_OF_FILE,
TOK_STRING,
TOK_ISTRING,
TOK_NAME,
TOK_VARARG,
TOK_NEWLINE,
TOK_ESCAPED_NEWLINE,
TOK_SPACE,
TOK_SPECIAL,
TOK_INTEGER,
TOK_BEGIN_INSTR,
TOK_ALIGN_ADDR,
TOK_ALIGN_MULT,
TOK_ORG,
TOK_PAGE,
TOK_PAGE_START,
TOK_NEWPAGE,
TOK_EQU,
TOK_EXPORT,
TOK_BEGIN_SECTION,
TOK_END_SECTION,
TOK_SHARP,
TOK_BLOCK,
TOK_FILL,
TOK_SET_PAGE,
TOK_PC_FAIL,
TOK_PC_FAIL1,
TOK_PC_SUCCESS,
TOK_BACKSLASH,
TOK_EXPR,
TOK_LIST,
TOK_END_FILE,
TOK_OP_IFTHEN,
TOK_OP_ELSE,
TOK_OP_LOG_OR,
TOK_OP_LOG_AND,
TOK_OP_LOG_XOR,
TOK_OP_LOG_NOT,
TOK_OP_BIN_OR,
TOK_OP_BIN_AND,
TOK_OP_BIN_XOR,
TOK_OP_BIN_NOT,
TOK_OP_SHIFT_LEFT,
TOK_OP_SHIFT_RIGHT,
TOK_OP_EQU,
TOK_OP_NEQ,
TOK_OP_LT,
TOK_OP_GT,
TOK_OP_LE,
TOK_OP_GE,
TOK_OP_CMP,
TOK_OP_ADD,
TOK_OP_SUB,
TOK_OP_MUL,
TOK_OP_DIV,
TOK_OP_MOD,
TOK_OP_POW,
TOK_DOT,
TOK_PAREN_OPEN,
TOK_PAREN_CLOSE,
TOK_BRACE_OPEN,
TOK_BRACE_CLOSE,
TOK_SQUARE_OPEN,
TOK_SQUARE_CLOSE,
TOK_COMMA,
TOK_PASTE,
TOK_DEFINED,
TOK_DEFINE,
TOK_MACRO,
TOK_REDEF,
TOK_UNDEF,
TOK_ENDM,
TOK_PP_BEGIN_SECTION,
TOK_PP_END_SECTION,
TOK_INCLUDE,
TOK_IF,
TOK_THEN,
TOK_IFDEF,
TOK_IFNDEF,
TOK_ELSE,
TOK_ELSEIF,
TOK_ELSEIFDEF,
TOK_ELSEIFNDEF,
TOK_ENDIF,
TOK_FOR,
TOK_IN,
TOK_FROM,
TOK_TO,
TOK_STEP,
TOK_DO,
TOK_ENDFOR,
TOK_ERROR,
TOK_WARNING,
TOK_DEBUG,
TOK_FEED_ACTION,
TOK_FEED_LIST,
TOK_OUTPUT,

    TOKENS_COUNT
  };

struct mult_list_t
{
  int mult;
  struct list_t *list;
};

struct page_addr_t
{
  int page;
  int addr;
};

struct position_t
{
  char *file;
  int line_begin;
  int line_end;
  struct token_t *original;
};

struct token_t
{
  struct position_t position;
  int type;
  // will be auto-generated
  union
  {
struct mult_list_t t_struct_mult_list_t;
struct processor_t* t_struct_processor_t_p;
struct proc_instr_t* t_struct_proc_instr_t_p;
struct proc_instr_arg_t* t_struct_proc_instr_arg_t_p;
int t_int;
int* t_int_p;
char* t_char_p;
char t_char;
struct page_addr_t t_struct_page_addr_t;
struct expr_t* t_struct_expr_t_p;
struct list_t* t_struct_list_t_p;
struct action_t* t_struct_action_t_p;

  } value;
};

struct token_t *token_make_pointer(struct token_t token);
struct token_t *token_copy_pointer(struct token_t *token);
void token_release_pointer(struct token_t *token);

#define TOKEN_POSITION_HERE(ORIGINAL) token_position_fill(string_new(__func__),__LINE__,__LINE__,token_copy_pointer(ORIGINAL))
#define TOKEN_POSITION_HERE_NO_ORIG token_position_fill(string_new(__func__),__LINE__,__LINE__,NULL)
struct position_t token_position_fill(char *file, int line_begin, int line_end, struct token_t *original);
struct position_t token_position_copy(struct position_t pos);
struct position_t token_position_set_original(struct position_t pos, struct token_t *original);
struct position_t token_position_set_original_longest(struct position_t pos, struct token_t o1, struct token_t o2);
void token_position_free(struct position_t pos);

struct token_t token_copy(struct token_t token);
void token_print(struct token_t token, FILE *file);
void token_print_nowarning(struct token_t token, FILE *file);
int token_value_signed_check(int value, int mask);
int token_value_unsigned_check(int value, int mask);
void token_free(struct token_t token);

#endif
